@@ -13,7 +13,7 @@ int fsetfilecon_raw(int fd, security_context_t context)
 			 0);
 }
 
-hidden_def(setfilecon_raw)
+hidden_def(fsetfilecon_raw)
 
 int fsetfilecon(int fd, security_context_t context)
 {
